-- Multi-Wave
-- Layer up to 4 sine waves onto the matrix

-- Define properties
property("Background", COLOUR, 0,0,0) -- Background Colour
property("Wave1", GRADIENT, 0,0,0,0,0.5,255,0,0,1,0,0,0) -- Wave 1 colour
property("Phase1",INTEGER,0,0,359) -- Phase offset of sine wave 1
property("Amp1",FLOAT,1,0,1) -- Amplitude of wave 1 (0-1)
property("Freq1",INTEGER,1,0,359) -- Frequency of wave 1
property("Active1", BOOLEAN,true) -- Should wave 1 be displayed?
property("Thickness1",INTEGER, 5,1,100) -- Thickness of line for wave 1
property("Reverse1",BOOLEAN,false)
property("Wave2", GRADIENT, 0,0,0,0,0.5,0,255,0,1,0,0,0) -- Wave 2 colour
property("Phase2",INTEGER,90,0,359) -- Phase offset of sine wave 2
property("Amp2",FLOAT,1,0,1) -- Amplitude of wave 2 (0-1)
property("Freq2",INTEGER,1,0,359) -- Frequency of wave 2
property("Active2", BOOLEAN,false) -- Should wave 2 be displayed?
property("Thickness2",INTEGER, 5,1,100) -- Thickness of line for wave 2
property("Reverse2",BOOLEAN,false)
property("Wave3", GRADIENT, 0,0,0,0,0.5,0,0,255,1,0,0,0) -- Wave 3 colour
property("Phase3",INTEGER,180,0,359) -- Phase offset of sine wave 3
property("Amp3",FLOAT,1,0,1) -- Amplitude of wave 3 (0-1)
property("Freq3",INTEGER,1,0,359) -- Frequency of wave 3
property("Active3", BOOLEAN,false) -- Should wave 3 be displayed?
property("Thickness3",INTEGER, 5,1,100) -- Thickness of line for wave 3
property("Reverse3",BOOLEAN,false)
property("Wave4", GRADIENT, 0,0,0,0,0.5,255,255,0,1,0,0,0) -- Wave 4 colour
property("Phase4",INTEGER,270,0,359) -- Phase offset of sine wave 4
property("Amp4",FLOAT,1,0,1) -- Amplitude of wave 4 (0-1)
property("Freq4",INTEGER,1,0,359) -- Frequency of wave 4
property("Active4", BOOLEAN,false) -- Should wave 4 be displayed?
property("Thickness4",INTEGER, 5,1,100) -- Thickness of line for wave 4
property("Reverse4",BOOLEAN,false)


-- Define pixel function
function pixel(frame, x, y)
    wave = {} -- Initialise table to store waves that the pixel falls into
    -- Calculate the Y value for each wave for the X position
    for i = 1,4 do
        if _G["Reverse"..i] then
            _G["p"..i] = round((math.sin(((x/(width*(1/_G["Freq"..i])))+((frames-frame)/frames)+(_G["Phase"..i]/360))*2*math.pi)/2)*(height*_G["Amp"..i])+(((height-1*_G["Amp"..i]))/2))
        else
            _G["p"..i] = round((math.sin(((x/(width*(1/_G["Freq"..i])))+(frame/frames)+(_G["Phase"..i]/360))*2*math.pi)/2)*(height*_G["Amp"..i])+(((height-1*_G["Amp"..i]))/2))
        end
    end
    -- Store wave number in wave list if the pixel falls within the thickness of the wave (and the wave is active)
    if dist(x,p1,x,y) <= Thickness1/2 and Active1 then
        wave[#wave+1] = 1
    end
    if dist(x,p2,x,y) <= Thickness2/2 and Active2 then
        wave[#wave+1] = 2
    end
    if dist(x,p3,x,y) <= Thickness3/2 and Active3 then
        wave[#wave+1] = 3
    end
    if dist(x,p4,x,y) <= Thickness4/2 and Active4 then
        wave[#wave+1] = 4
    end

    -- Determine the required colour based on the number of waves the pixel falls in (allows overlapping waves)
    if #wave == 1 then -- 1 wave, so output the colour from the gradient based on the distance from the centre of the wave
        out = _G["Wave"..wave[1]]:lookup((y-(_G["p"..wave[1]]-(_G["Thickness"..wave[1]]/2)))/_G["Thickness"..wave[1]])
    elseif #wave == 0 then -- 0 waves, so output background colour
        out = Background
    else -- 2 or more waves, so additively mix the output colours
        C1 = _G["Wave"..wave[#wave-1]]:lookup((y-(_G["p"..wave[#wave-1]]-(_G["Thickness"..wave[#wave-1]]/2)))/_G["Thickness"..wave[#wave-1]])
        C2 = _G["Wave"..wave[#wave]]:lookup((y-(_G["p"..wave[#wave]]-(_G["Thickness"..wave[#wave]]/2)))/_G["Thickness"..wave[#wave]])

        out = colour.new(math.max(C1.red,C2.red),math.max(C1.green,C2.green),math.max(C1.blue,C2.blue))
    end
    return out -- output the appropriate colour
end

-- Simple rounding function to round to nearest integer
function round(num)
    local mult = 10^(0)
    return math.floor(num * mult + 0.5) / mult
end

--[[
Copyright 2016 Pharos Architectural Controls Limited. All rights reserved.

This sample content is made freely available for illustrative purposes only. Pharos Architectural Controls grants a non-exclusive copyright licence to use this sample content from which you can generate content tailored to your own specific needs for use with products from Pharos Architectural Controls. This sample content has not been thoroughly tested under all conditions and Pharos Architectural Controls cannot guarantee reliability or serviceability.

This sample content is provided "as is" without warranties of any kind. The implied warranties of non-infringement, merchantability and fitness for a particular purpose are expressly disclaimed. In no event shall Pharos Architectural Controls be liable to any party for direct, indirect, incidental or consequential damages arising from the use of this sample content.

For further information please contact support@pharoscontrols.com.
--]]